# -*- coding: utf-8 -*-
"""
    lantz.qt.blocks
    ~~~~~~~~~~~~~~~

    Building blocks for rich applications.

    :copyright: 2018 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

from .loop import Loop, LoopUi
from .scan import Scan, ScanUi
from .feat import Feat, FeatUi
from .featscan import FeatScan, FeatScanUi
from .chart import ChartUi
from .layouts import HorizonalUi, VerticalUi, ToolbarLeftRightUi
from .wlayout import BorderUi