# -*- coding: utf-8 -*-
"""
    lantz.qt.log
    ~~~~~~~~~~~~

    Implements base class for graphical applications.

    :copyright: 2018 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""


from lantz.core.log import get_logger

LOGGER = get_logger('lantz.qt')